@@ -2,11 +2,14 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+/*! Definition for low level error class for core methods !*/
+
 use std::{self, path::PathBuf};
 
 use crate::core::deviceinfo::DeviceInfo;
 
 #[derive(Clone, Debug)]
+/// Internal error for low-level devicemapper operations
 pub enum Error {
     /// An error returned on failure to create a devicemapper context
     ContextInit(String),
@@ -20,7 +23,12 @@ pub enum Error {
     /// An error returned exclusively by DM methods.
     /// This error is initiated in DM::do_ioctl and returned by
     /// numerous wrapper methods.
-    Ioctl(Option<Box<DeviceInfo>>, Box<nix::Error>),
+    Ioctl(
+        u8,
+        Option<Box<DeviceInfo>>,
+        Option<Box<DeviceInfo>>,
+        Box<nix::Error>,
+    ),
 
     /// An error returned when the response exceeds the maximum possible
     /// size of the ioctl buffer.
@@ -31,19 +39,21 @@ pub enum Error {
 
     /// An error returned on general IO failure
     GeneralIo(String),
+
+    /// An error synchronizing with udev
+    UdevSync(String),
 }
 
 impl std::fmt::Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             Error::ContextInit(err) => {
-                write!(f, "DM context not initialized due to IO error: {}", err)
+                write!(f, "DM context not initialized due to IO error: {err}")
             }
-            Error::InvalidArgument(err) => write!(f, "invalid argument: {}", err),
-            Error::Ioctl(hdr, err) => write!(
+            Error::InvalidArgument(err) => write!(f, "invalid argument: {err}"),
+            Error::Ioctl(op, hdr_in, hdr_out, err) => write!(
                 f,
-                "low-level ioctl error due to nix error; header result: {:?}, error: {}",
-                hdr, err
+                "low-level ioctl error due to nix error; ioctl number: {op}, input header: {hdr_in:?}, header result: {hdr_out:?}, error: {err}"
             ),
             Error::IoctlResultTooLarge => write!(
                 f,
@@ -57,7 +67,10 @@ impl std::fmt::Display for Error {
                 err
             ),
             Error::GeneralIo(err) => {
-                write!(f, "failed to perform operation due to IO error: {}", err)
+                write!(f, "failed to perform operation due to IO error: {err}")
+            }
+            Error::UdevSync(err) => {
+                write!(f, "failed to perform udev sync operation: {}", err)
             }
         }
     }
@@ -66,7 +79,7 @@ impl std::fmt::Display for Error {
 impl std::error::Error for Error {
     fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
         match self {
-            Error::Ioctl(_, err) => Some(err),
+            Error::Ioctl(_, _, _, err) => Some(err),
             _ => None,
         }
     }
